#include <glib.h>
#if HAVE_CONFIG_H
#ifdef G_OS_WIN32
#include "config_win.h"
#else
#include <config.h>
#endif
#endif
#if defined(HAS_PYTHON)
#include <Python.h>
#endif
#include <sstream>
#include <cstdlib>
#include "AlibavaConfig.h"
#include "GtkUtils.h"
#include "PluginHook.h"
#include "utils.h"
#include <iostream>



void CalibrationGUI::user_init()
{
    ScanConfig C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();

    int2entry("txtStart", C.from());
    int2entry("txtEnd", C.to());
    int2entry("txtNpulse", C.npts());
    int2entry("txtSamples", C.nevts());
    int2entry("txtCalDelay", _cnf.delay());
    int2entry("txtCharge", _cnf.charge());

    if ( _cnf.scan_charge() )
        activate_button("btnScanVar", true );
    else
        activate_button("btnDelayScan", true );


    if (firmware()==0)
    {
        gtk_widget_set_sensitive( get_widget("txtCalDelay"), 0 );
        gtk_widget_set_sensitive( get_widget("btnDelayScan"), 0 );
    }

    g_signal_connect(G_OBJECT(get_widget("btnScanVar")),"toggled", G_CALLBACK(on_scan_changed),this);
}

void CalibrationGUI::process_text(const std::string &s, GtkWidget *w)
{
    ScanConfig C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();
    bool changed = false;
    if (s=="txtSamples")
    {
        C.nevts( entry2int(s) );
        changed = true;
    }
    else
    {
        if (s=="txtEnd")
        {
            int max = btn_active("btnScanVar") ? 261375 : 255 ;
            int val = entry2int(s);
            if (val>max)
                val = max;

            C.to( val );
            changed = true;
        }
        else if (s=="txtNpulse")
        {
            C.npts( entry2int(s) );
            if ( C.npts()>255 )
                C.npts(255);

            changed = true;
        }
        else if (s=="txtStart")
        {
            int val = entry2int(s, 0);
            if (val<0)
                val = 0;
            C.from( val );
            changed = true;
        }
        else if (s=="txtCalDelay")
        {
            _cnf.delay( entry2int(s) );
        }
        else if (s=="txtCharge")
        {
            _cnf.charge( entry2int(s) );
        }

    }
    if (changed)
    {
        C.adjust();
        if (_cnf.scan_charge())
            _cnf.charge_scan(C);
        else
            _cnf.delay_scan(C);
    }    check_values();

    C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();
    int2entry("txtStart", C.from());
    int2entry("txtEnd", C.to());
    int2entry("txtNpulse", C.npts());
    int2entry("txtSamples", C.nevts());
    int2entry("txtCalDelay", _cnf.delay());
    int2entry("txtCharge", _cnf.charge());
}

void CalibrationGUI::check_values()
{
    ScanConfig C = _cnf.scan_charge() ? _cnf.charge_scan() : _cnf.delay_scan();
    C.to( entry2int("txtEnd") );
    C.from( entry2int("txtStart") );
    C.npts(entry2int("txtNpulse") );

    _cnf.scan_charge( btn_active("btnScanVar") );
    C.set_min_step( _cnf.scan_charge() ? 1024 : 1 );

    unsigned int max = _cnf.scan_charge() ? 261375 : 255 ;
    if (C.to() > max)
        C.to( max );

    if ( C.from() < 0 )
        C.from(0);

    if ( C.npts()>255 )
        C.npts(255);

    C.adjust();

    if (_cnf.scan_charge())
        _cnf.charge_scan(C);
    else
        _cnf.delay_scan(C);
}

void CalibrationGUI::set_values()
{
    check_values();
    _cnf.charge( entry2int("txtCharge") );
    _cnf.delay( entry2int("txtCalDelay") );
    _cnf.scan_charge( btn_active("btnScanVar") );
}

void CalibrationGUI::on_scan_changed(GtkWidget *W, CalibrationGUI *cal)
{
    ScanConfig C =  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(W) ) ? cal->_cnf.charge_scan() : cal->_cnf.delay_scan();
    cal->int2entry("txtStart", C.from());
    cal->int2entry("txtEnd", C.to());
    cal->int2entry("txtNpulse", C.npts());
    cal->int2entry("txtSamples", C.nevts());
}

void CalibrationGUI::auto_connect()
{
   // SET_CONNECT(on_scan_changed);
}

void LaserSyncGUI::user_init()
{
    int2spin("spinFrom", _cnf.from());
    int2spin("spinTo", _cnf.to());
    int2spin("spinStep", _cnf.step());
    int2entry("txtSamples", _cnf.nevts());
}

void LaserSyncGUI::set_values()
{
    _cnf.from(spin2int("spinFrom") );
    _cnf.to(spin2int("spinTo") );
    _cnf.set_npts(spin2int("spinStep") );
    _cnf.nevts(entry2int("txtSamples") );
}

void BeetleConfigGUI::user_init()
{
    BeetleConfig::iterator ip;
    for (ip=_cnf.begin(); ip!=_cnf.end(); ++ip)
    {
        std::ostringstream ostr;
        ostr << "scale" << ip->first;
        const char *wname = ostr.str().c_str();
        GtkWidget *ww = get_widget(wname);
        GtkWidget *spn = get_widget(ip->first);
        if (ww)
        {
            gtk_range_set_adjustment(GTK_RANGE(ww),
                                     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spn)));

            gtk_spin_button_set_value(GTK_SPIN_BUTTON(spn), 0xff & ip->second);
            gtk_widget_queue_draw(ww);
        }
        else
            int2entry(ip->first, 0xff & ip->second);

        gtk_widget_queue_draw(spn);
    }
    activate_button("btnMaskChip0", _cnf.mask() & 1);
    activate_button("btnMaskChip1", _cnf.mask() & 2);

    if (firmware()==0)
    {
        gtk_widget_set_sensitive( get_widget("btnMaskChip0"), 0 );
        gtk_widget_set_sensitive( get_widget("btnMaskChip1"), 0 );
    }

}

void BeetleConfigGUI::set_values()
{
    BeetleConfig::iterator ip;
    int ival = 0;
    //printf("Setting Beetle values\n");
    for (ip=_cnf.begin(); ip!=_cnf.end(); ++ip)
    {
        unsigned short addr = _cnf[ip->first] & 0xff00;
        ip->second = addr | (entry2int(ip->first) & 0xff);
        //printf("%15s %x: add %8x value %8x res %8x\n", ip->first, ival, addr, entry2int(ip->first), ip->second);
        ival ++;
    }
    short mask = 0;
    if ( btn_active("btnMaskChip0") )
        mask |= 1;
    if ( btn_active("btnMaskChip1") )
        mask |= 2;

    _cnf.mask(mask);
}

void LaserConfigGUI::user_init()
{
    int2spin("spinDelay", _cnf.delay());
}

void LaserConfigGUI::set_values()
{
    _cnf.delay(spin2int("spinDelay"));
}

void TriggerConfigGUI::user_init()
{
    const char *spins[] = { "spinDACa", "spinDACb", "spinDACc", "spinDACd", 0 };
    const char *scales[] = { "scaleDACa", "scaleDACb", "scaleDACc", "scaleDACd", 0 };
    int i;
    for (i=0;spins[i]; i++)
    {
        gtk_range_set_adjustment(GTK_RANGE(get_widget(scales[i])),
                                 gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget(spins[i])))
                             );
    }
    click_button(_cnf.trigger_and() ? "rbtnAND" : "rbtnOR");
    click_button(_cnf.pulse() ? "rbtnTriggerPulse" : "rbtnTriggerIn");
    int2spin("spinDACa", _cnf.dacA() );
    int2spin("spinDACb", _cnf.dacB() );
    int2spin("spinDACc", _cnf.dacC() );
    int2spin("spinDACd", _cnf.dacD() );
}

void TriggerConfigGUI::set_values()
{
    _cnf.trigger_and(btn_active("rbtnAND") );
    _cnf.pulse(btn_active("rbtnTriggerPulse") );
    _cnf.dacA(spin2int("spinDACa") );
    _cnf.dacB(spin2int("spinDACb") );
    _cnf.dacC(spin2int("spinDACc") );
    _cnf.dacD(spin2int("spinDACd") );
}

void DAQConfigGUI::user_init()
{
    int2entry("txtMonChannel", _cnf.chan());
    int2entry("txtSampleSize", _cnf.sample_size());
    int2entry("entryMaxEvents", _cnf.nevts());
    int2entry("txtDelay", _cnf.delay());
    if (firmware()<2)
    {
        gtk_widget_set_sensitive( get_widget("btnEnableBusy"), 0);
        gtk_widget_set_sensitive( get_widget("btnPulseShape"), 0);
    }
    else
    {
        activate_button("btnPulseShape", _cnf.pulse_shape());
        activate_button("btnEnableBusy", _cnf.enable_busy());
    }
}

void DAQConfigGUI::set_values()
{
    _cnf.chan(entry2int("txtMonChannel") );
    _cnf.sample_size(entry2int("txtSampleSize") );
    _cnf.nevts( entry2int("entryMaxEvents"));
    _cnf.delay( entry2int("txtDelay"));
    _cnf.enable_busy( btn_active("btnEnableBusy") );
    _cnf.pulse_shape( btn_active("btnPulseShape"));
}

void DAQConfigGUI::process_text(const std::string &s, GtkWidget *w)
{
    if (s=="entryMaxEvents")
    {
        int uval = 0;
        if ( s.empty() )
        {
            uval = G_MAXINT;
        }
        else
        {
            uval = entry2int(s, G_MAXINT);
        }
        _cnf.nevts(uval);
    }
}


void AnalysisConfigGUI::user_init()
{
    double2entry("txtPedWeight", _cnf.ped_weight());
    double2entry("txtNoiseWeight", _cnf.noise_weight());
    double2entry("txtSeedCut", _cnf.seed_cut());
    double2entry("txtNeighCut", _cnf.neigh_cut());
    double2entry("txtSNCut", _cnf.sn_cut());
    click_button(_cnf.do_cmmd() ? "btnCmmdActive" : "btnCmmdInactive");
    if ( _cnf.polarity() == 0.0)
        click_button("btnXXPolarity");
    else
    {
        if (_cnf.polarity()>0.0)
            click_button("btnPosPolarity");
        else
            click_button("btnNegPolarity");
    }
}



void AnalysisConfigGUI::on_activate_common_mode(GtkWidget *w, AnalysisConfigGUI *A)
{

}

void AnalysisConfigGUI::auto_connect()
{
    SET_CONNECT(on_activate_common_mode);
}

void AnalysisConfigGUI::set_values()
{
    _cnf.set_weights(entry2double("txtPedWeight"), entry2double("txtNoiseWeight"));
    _cnf.seed_cut(entry2double("txtSeedCut"));
    _cnf.neigh_cut(entry2double("txtNeighCut"));
    _cnf.sn_cut(entry2double("txtSNCut"));
    _cnf.do_cmmd(btn_active("btnCmmdActive"));

    double polarity = 0.0;
    if ( btn_active("btnXXPolarity") )
        polarity = 0.0;
    else
    {
        if ( btn_active("btnPosPolarity"))
            polarity = 1.0;
        else
            polarity = -1.0;
    }
    _cnf.polarity( polarity );
}

void PluginConfigGUI::auto_connect()
{
    SET_CONNECT(on_browse_lib);
    SET_CONNECT(on_browse_path);
    SET_CONNECT(on_find_symbols);
}

void PluginConfigGUI::on_find_symbols(GtkWidget *, PluginConfigGUI *P)
{
    P->set_values();

    char *libname = P->_cnf.lib_path();
    PluginHook *ph = new PluginHook(libname);
    ph->init( P->get_xml_buffer(), P->get_buffer_size(), _("hook_dialog"));
    int rc = gtk_dialog_run(GTK_DIALOG(ph->get_main_window()));
    if (rc == GTK_RESPONSE_OK)
    {
        gchar *dirname, *basename;
        const char *filename=ph->lib().c_str();
        const char *hook;
        dirname = g_path_get_dirname(filename);
        basename = g_path_get_basename(filename);
        hook = ph->hook().c_str();
        P->_cnf.lib(basename);
        P->_cnf.path(dirname);
        P->_cnf.hook(hook);
        P->user_init();
        g_free(dirname);
        g_free(basename);
    }
    g_free(libname);
    delete ph;
}

void PluginConfigGUI::on_browse_lib(GtkWidget *, PluginConfigGUI *ps)
{
#ifdef G_OS_WIN32
    const char *filters[] = {"C/C++ libs|*.dll", "Python scripts|*.py","All|*", 0};
#else G_OS_UNIX
    const char *filters[] = {"C/C++ (Linux) libs|*.so", "C/C++ (Mac OSX) libs|*.dylib", "Python scripts|*.py","All|*", 0};
#endif
    std::string ofile = ::get_file_name(_("Find shared library"), a_OpenFile, ps->_cnf.path().c_str(), filters);
    if (!ofile.empty())
    {
        gchar *dirname, *basename;
        const char *filename = ofile.c_str();
        dirname = g_path_get_dirname(filename);
        basename = g_path_get_basename(filename);
        ps->set_text("txtLibName", basename);
        ps->set_text("txtLibPath", dirname);
        ps->_cnf.path(dirname);
        ps->_cnf.lib(basename);
        if (g_str_has_suffix(basename,".py"))
        {
            ps->_cnf.lang("python");
            ps->activate_button("btnPython", true);
        }
        else if  (g_str_has_suffix(basename,".so"))
        {
            ps->_cnf.lang("c++");
            ps->activate_button("btnCpp", true);
        }
        g_free(basename);
        g_free(dirname);
    }
}

void PluginConfigGUI::on_browse_path(GtkWidget *, PluginConfigGUI *ps)
{
    std::string ofile = ::get_file_name(_("Find shared library folder"), a_OpenFolder, 0, 0);
    if (!ofile.empty())
    {
        ps->set_text("txtLibPath", ofile);
        ps->_cnf.path(ofile);
    }
}

void PluginConfigGUI::user_init()
{
    set_text("txtLibPath", _cnf.path());
    set_text("txtLibName", _cnf.lib());
    set_text("txtHook", _cnf.hook());
    if ( _cnf.lang() == "python" )
    {
        activate_button("btnPython", true);
    }
    else
        activate_button("btnCpp", true);
}

void PluginConfigGUI::set_values()
{
    _cnf.path( get_text("txtLibPath") );
    _cnf.lib( get_text("txtLibName") );
    _cnf.hook( get_text("txtHook"));
    if ( btn_active("btnPython") )
    {
        _cnf.lang( "python" );
    }
    else
    {
        _cnf.lang("c++");
    }
}


void ComparatorConfigGUI::auto_connect()
{
    SET_CONNECT(on_enable);
    SET_CONNECT(btn_mask_all);
    SET_CONNECT(btn_clear_all);
    SET_CONNECT(btn_reset_trim);
    SET_CONNECT(on_new_chip);
}

void ComparatorConfigGUI::user_init()
{
    GtkWidget *btn = get_widget("toggleComparator");
    if (_cnf.isEnabled())
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Disable Comparator");
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Enable Comparator");
    }
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(btn), _cnf.isEnabled() );
    gtk_widget_set_sensitive( get_widget("tableComparator"), _cnf.isEnabled() );
    gtk_widget_set_sensitive( get_widget("notebookComparator"), _cnf.isEnabled() );
    gtk_widget_set_sensitive( get_widget("tableThrs"), _cnf.isEnabled() );


    gtk_combo_box_set_active(GTK_COMBO_BOX(get_widget("chipCombo")), 0);
    gtk_range_set_adjustment(GTK_RANGE(get_widget("scaleThrs")),
                                     gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinThrs"))));

    gtk_range_set_adjustment(GTK_RANGE(get_widget("scaleIcomp")),
                                         gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinIcomp"))));


    GtkAdjustment *adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinIcomp")));
    gtk_signal_connect(GTK_OBJECT(adj),
                       "value-changed",
                       GTK_SIGNAL_FUNC(icomp_changed), this);


    adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(get_widget("spinThrs")));
    gtk_signal_connect(GTK_OBJECT(adj),
                       "value-changed",
                       GTK_SIGNAL_FUNC(thrs_changed), this);

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnPos")), _cnf.getPolarity());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(get_widget("btnTrack")),
    		_cnf.getMode()==ComparatorConfig::trackMode ? 1 : 0);

    on_new_chip(0, this);

}

void ComparatorConfigGUI::thrs_changed(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    double val = gtk_adjustment_get_value(GTK_ADJUSTMENT(w));
    C->_cnf.setMainTh(ichip, (unsigned short)val);
}
void ComparatorConfigGUI::icomp_changed(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    double val = gtk_adjustment_get_value(GTK_ADJUSTMENT(w));
    C->_cnf.setIcomp(ichip, (unsigned short)val);
}


void ComparatorConfigGUI::on_enable(GtkWidget *W, ComparatorConfigGUI *C)
{
    GtkWidget *btn = C->get_widget("toggleComparator");
    if (C->btn_active("toggleComparator"))
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Disable Comparator");
    }
    else
    {
        gtk_button_set_label(GTK_BUTTON(btn), "Enable Comparator");
    }
    C->_cnf.setEnable( C->btn_active("toggleComparator") );
    gtk_widget_set_sensitive( C->get_widget("tableComparator"), C->_cnf.isEnabled() );
    gtk_widget_set_sensitive( C->get_widget("notebookComparator"), C->_cnf.isEnabled() );
    gtk_widget_set_sensitive( C->get_widget("tableThrs"), C->_cnf.isEnabled() );
}

void ComparatorConfigGUI::set_values()
{
    _cnf.setEnable( btn_active("toggleComparator") );

    int iPol = btn_active("btnNeg") ? 0 : 1;
    _cnf.setPolarity(iPol);

    ComparatorConfig::CompMode iMode = btn_active("btnTrack") ? ComparatorConfig::trackMode : ComparatorConfig::pulseMode;
    _cnf.setMode(iMode);

}

void ComparatorConfigGUI::process_text(const std::string &s, GtkWidget *w)
{
    double val;
    GtkWidget *cmb = get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));


    if (s == "txtDelta")
    {
        val = entry2double(s, 0.0);
        if (val < 0.0)
        {
            double2entry(s, 0.0);
            val = 0.0;
        }
        else if (val > 255)
        {
            double2entry(s, 255);
            val = 255.0;
        }
        else
        {
            //val = int(val/7.8125e-3)*7.8125e-3;
            //double2entry(s, val);
        }
        _cnf.setDeltaTh(ichip, val);
    }
    else if (s == "txtVrs")
    {
        val = entry2double(s, 0.0);
        if (val < 0.0)
        {
            double2entry(s, 0.0);
            val = 0.0;
        }
        else if (val > 255)
        {
            double2entry(s, 255);
            val = 255;
        }
        else
        {
            //val = int(val/0.0048828125)*0.0048828125;
            //double2entry(s, val);
        }
        _cnf.setVrc(ichip, val);
    }
}

void ComparatorConfigGUI::btn_mask_all(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    for (int j=0; j<128; j++)
        C->_cnf.setMask(ichip, j, true);

    on_new_chip(cmb, C);
}

void ComparatorConfigGUI::btn_clear_all(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    for (int j=0; j<128; j++)
        C->_cnf.setMask(ichip, j, false);

    on_new_chip(cmb, C);
}

void ComparatorConfigGUI::on_new_chip(GtkWidget *w, ComparatorConfigGUI *C)
{
    int ichip;
    if ( w )
        ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(w));
    else
        ichip = 0;

    C->double2entry("txtDelta", C->_cnf.getDeltaTh(ichip));
    C->double2entry("txtVrs", C->_cnf.getVrc(ichip));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(C->get_widget("spinThrs")), C->_cnf.getMainTh(ichip));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(C->get_widget("spinIcomp")), C->_cnf.getIcomp(ichip));



    GtkTable *tbl;

    tbl = GTK_TABLE(C->get_widget("tableMask"));
    C->draw_table( tbl, ichip, 0 );

    tbl = GTK_TABLE(C->get_widget("tableTrim"));
    C->draw_table( tbl, ichip, 1 );

}

void ComparatorConfigGUI::draw_table(GtkTable *tbl, int ichip, int k)
{

    GtkWidget *T = GTK_WIDGET(tbl);
    // Now the heavy part... Resize the tables with the masks and test bits
    int i, ic, ir;
    int nchan = 128;
    int ncol = 16;
    int nrow = nchan / ncol + ( (nchan % ncol) ? 1 : 0);
    GtkWidget *tmp;
    char line[10];


    gtk_container_foreach(GTK_CONTAINER(T),
                          (GtkCallback) gtk_widget_destroy,
                          NULL);
    gtk_table_resize(tbl, nrow, ncol + 1);
    for (i = 0, ic = 0, ir = -1; i < nchan; i++, ic++)
    {
        if (!(i % ncol))
        {
            ir++;
            ic = 0;
            // add the label
            sprintf(line, "%d", i);
            tmp = gtk_label_new(line);
            gtk_widget_show(tmp);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);
            ic++;
        }
        if (k == 0)
        {
            // create the check button
            tmp = gtk_check_button_new();
            gtk_widget_show(tmp);
            gtk_toggle_button_set_mode(GTK_TOGGLE_BUTTON(tmp), FALSE);
            gtk_widget_set_size_request(tmp, 10, 10);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND), (GtkAttachOptions) (0), 0, 0);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), _cnf.getMask(ichip, i));
            gtk_signal_connect(GTK_OBJECT(tmp), "toggled",
                               GTK_SIGNAL_FUNC(btn_toggled), this);

        }
        else
        {
            sprintf(line, "%d", _cnf.getTrim(ichip, i));
            tmp = gtk_entry_new();
            gtk_entry_set_width_chars(GTK_ENTRY(tmp),3);
            gtk_entry_set_text(GTK_ENTRY(tmp), line);
            gtk_widget_set_size_request(tmp, 20, 20);
            gtk_widget_show(tmp);
            gtk_table_attach(tbl, tmp, ic, ic + 1, ir, ir + 1, (GtkAttachOptions) (GTK_EXPAND |GTK_FILL),
                             (GtkAttachOptions) (0), 0, 0);
            gtk_signal_connect(GTK_OBJECT(tmp), "changed",
                               GTK_SIGNAL_FUNC(txt_changed), this);

        }
        gtk_object_set_data(GTK_OBJECT(tmp), "chip", (gpointer) ichip);
        gtk_object_set_data(GTK_OBJECT(tmp), "channel", (gpointer) i);
        gtk_object_set_data(GTK_OBJECT(tmp), "type", (gpointer) k);
    }
    gtk_widget_queue_resize(T);



}

void ComparatorConfigGUI::btn_toggled(GtkWidget *w, ComparatorConfigGUI *C)
{
    long ichip = long(gtk_object_get_data(GTK_OBJECT(w), "chip"));
    long ichan = long(gtk_object_get_data(GTK_OBJECT(w), "channel"));
    long itype = long(gtk_object_get_data(GTK_OBJECT(w), "type"));
    bool lval   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));

    C->_cnf.setMask(ichip, ichan, lval);
}

void ComparatorConfigGUI::txt_changed(GtkWidget *w, ComparatorConfigGUI *C)
{
    long ichip = long(gtk_object_get_data(GTK_OBJECT(w), "chip"));
    long ichan = long(gtk_object_get_data(GTK_OBJECT(w), "channel"));
    const char *cval = gtk_entry_get_text(GTK_ENTRY(w));

    char *endp;
    int ival = strtol(cval, &endp, 0);
    if (endp != cval )
    {
        ival &= 0x1f;
        C->_cnf.setTrim(ichip, ichan, ival);
    }
}

void ComparatorConfigGUI::btn_reset_trim(GtkWidget *w, ComparatorConfigGUI *C)
{
    GtkWidget *cmb = C->get_widget("chipCombo");
    int ichip = gtk_combo_box_get_active(GTK_COMBO_BOX(cmb));
    for (int j=0; j<128; j++)
        C->_cnf.setTrim(ichip, j, 0);

    on_new_chip(cmb, C);
}
