/* -*- mode: c++ -*- */
#ifndef __Analysis_h__
#define __Analysis_h__

#include <stack>
#include <vector>
#include <ConfigFile.h>

class Analysis : public ConfigFile
{
    public:
        typedef std::pair<int, double> Hit;
        typedef std::vector< Hit > HitList;
    private:
        double _ped[256];
        double _noise[256];
        double _hitmap[256];
        double _signal[256];
        double _raw[256];
        double _sn[256];
        double _header[2][16];
        double _ped_weight;
        double _noise_weight;
        double _cmmd[2];
        double _cmmd_noise[2];
        double _seedcut;
        double _neighcut;
        double _sncut;
        int    _mask;
        double _polarity;
        bool _do_cmmd;
        bool _update;
        
        HitList _hits;

        void cpy(const Analysis &a);
    public:
        Analysis();
        Analysis(const Analysis &s)
        {
            cpy(s);
        }
        virtual ~Analysis();
        void cpy_data(const Analysis &a);
        void reset(bool keep_pedestals=false);

        void set_weights(double ped, double noise)
        {
            _ped_weight = ped;
            _noise_weight = noise;

        }
        double ped_weight() const
        {
            return _ped_weight;
        }
        double noise_weight() const
        {
            return _noise_weight;
        }

        void process_event(const unsigned short *, int);
        void update_pedestals(const double *);
        void find_clusters();
        void common_mode();

        double *header(int i)
        {
            return _header[i];
        }

        double ped(int i) const
        {
            return _ped[i];
        }
        double noise(int i) const
        {
            return _noise[i];
        }
        double hits(int i) const
        {
            return _hitmap[i];
        }
        double signal(int i) const
        {
            return _signal[i];
        }
        double sn(int i) const
        {
            return _sn[i];
        }

        void ped(int i, double v)
        {
            _ped[i] = v;
        }
        void noise(int i, double v)
        {
            _noise[i] = v;
        }
        void signal(int i, double v)
        {
            _signal[i] = v;
        }
        void sn(int i, double v)
        {
            _sn[i] = v;
        }
        void hitmap(int i)
        {
            _hitmap[i]++;
        }

        void add_hit(int ch, double sg)
        {
            _hits.push_back(Hit(ch, sg) );
        }
        HitList::iterator begin()
        {
            return _hits.begin();
        }
        HitList::iterator end()
        {
            return _hits.end();
        }
        int nhits() const
        {
            return _hits.size();
        }
        bool empty() const
        {
            return _hits.empty();
        }
        const Hit &hit(int i) const
        {
            return _hits[i];
        }
        void clear()
        {
            _hits.clear();
        }

        double *pedestals()
        {
            return _ped;
        }
        double *noise()
        {
            return _noise;
        }
        double *hitmap()
        {
            return _hitmap;
        }
        double *event()
        {
            return _signal;
        }

        double *raw()
        {
            return _raw;
        }
        
        double *signal()
        {
            return _signal;
        }

        double *sn()
        {
            return _sn;
        }

        bool update_pedestals() const
        {
            return _update;
        }
        void update_pedestals(bool x)
        {
            _update = x;
        }

        double cmmd(int i) const
        {
            return _cmmd[i];
        }
        double cmmd_noise(int i) const
        {
            return _cmmd_noise[i];
        }

        double seed_cut() const
        {
            return _seedcut;
        }
        double neigh_cut() const
        {
            return _neighcut;
        }
        double sn_cut() const
        {
            return _sncut;
        }
        void seed_cut(double x)
        {
            _seedcut = x;
        }
        void neigh_cut(double x)
        {
            _neighcut=x;
        }
        void sn_cut(double x)
        {
            _sncut=x;
        }

        
        void do_cmmd(bool x) 
        {
            _do_cmmd = x;
        }
        bool do_cmmd() const
        {
            return _do_cmmd;
        }
        
        double polarity() const { return _polarity; }
        void polarity(double val);

        void load_data();
        void save_data();
};

#endif
