/*
 * ComparatorConfig.cc
 *
 *  Created on: Jun 4, 2013
 *      Author: lacasta
 */

#include "ComparatorConfig.h"

ChipCompConfig::ChipCompConfig()
    : ConfigFile("ChipConfig"),
      _mainth(0), _deltath(5), _Vrc(0), _icomp(5), _mask(128)
{
    for (int i=0; i<128; i++)
        _trim[i] = 0;
}

ChipCompConfig::ChipCompConfig(const ChipCompConfig &C)
: ConfigFile("ChipConfig"),
  _mainth(0), _deltath(0), _Vrc(0), _icomp(5), _mask(128)
{
    cpy(C);
}

ChipCompConfig &ChipCompConfig::operator=(const ChipCompConfig &C)
{
    if (this != &C)
        cpy(C);

    return *this;
}

void ChipCompConfig::cpy(const ChipCompConfig &C)
{
    _mainth = C._mainth;
    _deltath = C._deltath;
    _Vrc = C._Vrc;
    _icomp = C._icomp;
    _mask = C._mask;

    for (int i=0; i<128; i++)
        _trim[i] = C._trim[i];
}

void ChipCompConfig::load_data()
{
    if (exists())
    {
        setMainth( get_int("Ithmain", 0.0));
        setDeltath( get_int("Ithdelta", 0.0));
        setVrc( get_int("Vrs", 0.0));
        setIcomp( get_int("Icomp", 5.0));

        int iv, length;
        int *v = get_int_list("mask", length);
        for (iv=0; iv<length; iv++)
        {
          if (v[iv])
              _mask.Set(iv);
          else
              _mask.Clr(iv);
        }

        delete [] v;

        v = get_int_list("trim", length);
        for (iv=0; iv<length; iv++)
            _trim[iv] = v[iv];

        delete [] v;
    }
}

void ChipCompConfig::save_data()
{
    set_int("Ithmain", getMainth());
    set_int("Ithdelta", getDeltath());
    set_int("Vrs", getVrc());
    set_int("Icomp", getIcomp());

    int iv, v[128];
    for (iv=0; iv<128; iv++)
        v[iv] = _mask.Bit(iv) ? 1 : 0;

    set_int_list("mask", v, 128);

    for (iv=0; iv<128; iv++)
        v[iv] = _trim[iv];

    set_int_list("trim", v, 128);
}

void ChipCompConfig::setMask(const BitSet &b)
{
    if (b.nBits()==128)
        _mask = b;
}

ComparatorConfig::ComparatorConfig()
    : ConfigFile("comparator"), _enable(false),
      _polarity(1), _mode(trackMode)
{
    // TODO Auto-generated constructor stub
    chip[0].set_group("Chip.0");
    chip[1].set_group("Chip.1");
}

ComparatorConfig::ComparatorConfig(const ComparatorConfig &C)
{
    cpy(C);
}

ComparatorConfig &ComparatorConfig::operator =(const ComparatorConfig &C)
{
    if (this!=&C)
        cpy(C);

    return *this;
}

void ComparatorConfig::cpy(const ComparatorConfig &C)
{
    *dynamic_cast<ConfigFile *>(this) = *dynamic_cast<const ConfigFile *>(&C);
    _enable = C._enable;
    _mode = C._mode;
    _polarity = C._polarity;
    for (int ichip=0; ichip<2; ichip++)
        chip[ichip] = C.chip[ichip];
}

ComparatorConfig::~ComparatorConfig()
{
    // TODO Auto-generated destructor stub
}

void ComparatorConfig::load_data()
{
    if (exists())
    {
        setEnable( get_bool("enable"));
        setPolarity( get_int("polarity"));
        setMode( (CompMode)get_int("mode") );
    }
    for (int ichip=0; ichip<2; ichip++)
    {
        if (chip[ichip].key_file() != key_file())
            chip[ichip].set_file(key_file());
        else
            chip[ichip].load_data();
    }
}

void ComparatorConfig::save_data()
{
    set_bool("enable", isEnabled());
    set_int("polarity", getPolarity());
    set_int("mode", getMode());
    for (int ichip=0; ichip<2; ichip++)
        chip[ichip].save_data();
}
