/*
 * ComparatorConfig.h
 *
 *  Created on: Jun 4, 2013
 *      Author: lacasta
 */

#ifndef COMPARATORCONFIG_H_
#define COMPARATORCONFIG_H_

#include <BitSet.h>
#include <ConfigFile.h>


class ChipCompConfig: public ConfigFile
{
    private:
        unsigned short _mainth;
        unsigned short _deltath;
        unsigned short _Vrc;
        unsigned short _icomp;
        unsigned char  _trim[128];
        BitSet _mask;

        void cpy(const ChipCompConfig &);
    public:
        ChipCompConfig();
        ChipCompConfig(const ChipCompConfig &);
        ChipCompConfig &operator=(const ChipCompConfig &);

        virtual ~ChipCompConfig() {};

        void load_data();
        void save_data();

        unsigned short getDeltath() const
        {
            return _deltath;
        }

        void setDeltath(unsigned short deltath)
        {
            _deltath = deltath;
        }

        unsigned short getMainth() const
        {
            return _mainth;
        }

        void setMainth(unsigned short mainth)
        {
            _mainth = mainth;
        }

        const BitSet &getMask() const { return _mask; }

        const bool getMask(int ichan) const
        {
            return _mask.Bit(ichan);
        }

        void setMask(int ichan, bool val)
        {
            if (val)
                _mask.Set(ichan);
            else
                _mask.Clr(ichan);
        }

        void setMask(const BitSet &b);

        const unsigned char *getTrim() const { return _trim; }

        const unsigned char getTrim(int ichan) const
        {
            return _trim[ichan];
        }
        void setTrim(int ichan, unsigned char val)
        {
            _trim[ichan] = val;
        }

        unsigned short getVrc() const
        {
            return _Vrc;
        }

        void setVrc(unsigned short vrc)
        {
            _Vrc = vrc;
        }

        unsigned short getIcomp() const
        {
            return _icomp;
        }

        void setIcomp(unsigned short icomp)
        {
            _icomp = icomp;
        }
};

class ComparatorConfig: public ConfigFile
{
    public:
        enum CompMode { trackMode=0, pulseMode };

    private:
        bool _enable;

        // These two are common to all the chips
        // Thresholds will be for each chip
        int _polarity;
        CompMode _mode;
        ChipCompConfig chip[2];

        void cpy( const ComparatorConfig &C );
    public:

        ComparatorConfig();
        ComparatorConfig(const ComparatorConfig &c);
        ComparatorConfig &operator=(const ComparatorConfig &C);
        virtual ~ComparatorConfig();

        void load_data();
        void save_data();

        CompMode getMode() const
        {
            return _mode;
        }

        void setMode(CompMode mode)
        {
            _mode = mode;
        }

        int getPolarity() const
        {
            return _polarity;
        }

        void setPolarity(int polarity)
        {
            _polarity = polarity;
        }

        bool isEnabled() const
        {
            return _enable;
        }

        void setEnable(bool enable)
        {
            _enable = enable;
        }

        unsigned short getDeltaTh(int ichip) const
        {
            return chip[ichip].getDeltath();
        }

        void setDeltaTh(int ichip, unsigned short deltath)
        {
            chip[ichip].setDeltath(deltath);
        }

        unsigned short getMainTh(int ichip) const
        {
            return chip[ichip].getMainth();
        }

        void setMainTh(int ichip, unsigned short mainth)
        {
            chip[ichip].setMainth(mainth);
        }

        unsigned short getVrc(int ichip) const
        {
            return chip[ichip].getVrc();
        }

        void setVrc(int ichip, unsigned short vrc)
        {
            chip[ichip].setVrc(vrc);
        }

        const BitSet &getMask(int ichip) const { return chip[ichip].getMask(); }

        bool getMask(int ichip, int ichan) const
        {
            return chip[ichip].getMask(ichan);
        }
        void setMask(int ichip, int ichan, bool val)
        {
            chip[ichip].setMask(ichan, val);
        }
        void setMask(int ichip, const BitSet &b)
        {
            chip[ichip].setMask(b);
        }
        unsigned char getTrim(int ichip, int ichan) const
        {
            return chip[ichip].getTrim(ichan);
        }
        void setTrim(int ichip, int ichan, unsigned char val)
        {
            chip[ichip].setTrim(ichan, val & 0x1f);
        }
        const unsigned char *getTrim( int ichip ) const { return chip[ichip].getTrim(); }

        unsigned short getIcomp(int ichip) const { return chip[ichip].getIcomp(); }
        void setIcomp(int ichip, unsigned short val) { chip[ichip].setIcomp(val); }
};

#endif /* COMPARATORCONFIG_H_ */
