#include "DAQConfig.h"


DAQConfig::DAQConfig() :
ConfigFile("DAQ"), _chan(60), _nevt_per_sample(100), _nevts(10000),
_delay(60)
{
}
DAQConfig::DAQConfig(const DAQConfig &c)
{
    cpy(c);
}

DAQConfig::~DAQConfig()
{
}

void DAQConfig::cpy(const DAQConfig &c)
{
    *dynamic_cast<ConfigFile *>(this) = *dynamic_cast<const ConfigFile *>(&c);
    chan(c._chan);
    sample_size( c._nevt_per_sample );
    nevts( c._nevts );
    delay( c._delay );
    _fw_config = c._fw_config;
}

DAQConfig &DAQConfig::operator=(const DAQConfig &b)
{
    if ( this!=&b)
        cpy(b);

    return *this;
}

int DAQConfig::chan() const
{
    return _chan;
}
void DAQConfig::chan(int x)
{
    _chan = x;
    set_int("monitor_channel", x);
}

int DAQConfig::sample_size() const
{
    return _nevt_per_sample;
}
void DAQConfig::sample_size(int x)
{
    _nevt_per_sample = x;
    set_int("sample_size", x);
}
int DAQConfig::nevts() const
{
    return _nevts;
}

void DAQConfig::nevts(int x)
{
    _nevts = x;
    set_int("nevts", x);
}
int DAQConfig::delay() const
{
    return _delay;
}
void DAQConfig::delay(int x)
{
    _delay = x;
    set_int("delay", x);
}


void DAQConfig::load_data()
{
    if (exists())
    {
        chan(get_int("monitor_channel") );
        sample_size(get_int("sample_size") );
        nevts(get_int("nevts"));
        delay(get_int("delay"));
        enable_busy(get_bool("enable_busy"));
        pulse_shape(get_bool("pulse_shape"));
    }
}

void DAQConfig::save_data()
{
    set_int("monitor_channel", chan());
    set_int("sample_size", sample_size());
    set_int("nevts", nevts());
    set_int("delay", delay());
    set_bool("enable_busy", enable_busy());
    set_bool("pulse_shape", pulse_shape());
}

unsigned short DAQConfig::firmware_config() const
{
    return _fw_config;
}

void DAQConfig::pulse_shape(bool x)
{
    unsigned short bit = 2;
    if (x)
        _fw_config |= bit;
    else
        _fw_config &= ~bit;
}
bool DAQConfig::pulse_shape() const
{
    return _fw_config & 2;
}

void DAQConfig::enable_busy(bool x)
{
    unsigned short bit = 1;
    if (x)
        _fw_config |= bit;
    else
        _fw_config &= ~bit;
}
bool DAQConfig::enable_busy() const
{
    return _fw_config & 1;
}
