/*
 * LaserSyncRun.cc
 *
 *  Created on: May 12, 2010
 *      Author: lacasta
 */
#include <iostream>
#include "DAQmanager.h"
#include "LaserSyncRun.h"
#include "Analysis.h"
#include "ScanConfig.h"
#include <ConfigFactory.h>

LaserSyncRun::LaserSyncRun(DAQmanager *b)
    : RunManager(b)
{

}

LaserSyncRun::~LaserSyncRun()
{
}

void LaserSyncRun::start_of_run()
{
    ConfigFactory *cfg = ConfigFactory::instance();
    LaserSyncConfig *lc = dynamic_cast<LaserSyncConfig *>(cfg->get("lasersync"));
    Analysis *ana = dynamic_cast<Analysis *>(cfg->get("analysis"));

    ana->update_pedestals(false);

    DataMonitor *mon = get_daq_manager()->get_monitor();
    if (mon)
        mon->create_signal_profile("Delay",
                                   lc->npts(),
                                   lc->from(),
                                   lc->to() );

    get_daq_manager()->set_expected(lc->npts()*lc->nevts());
    get_daq_manager()->set_received(0);
}

int LaserSyncRun::loop(std::ostream &oheader)
{
    unsigned int i, val, step;
    ConfigFactory *cfg = ConfigFactory::instance();
    LaserSyncConfig *lc = dynamic_cast<LaserSyncConfig *>(cfg->get("lasersync"));

    step = lc->step();
    oheader << lc->npts() << "; "
            << lc->from() << "; "
            << lc->to()  << "; "
            << lc->step();

    int rc = 0;
    for (i=0, val=lc->from(); i<lc->npts();i++,val+=step)
    {
        if (get_daq_manager()->stop())
            break;

        check_new_event(i);

        if ( (rc=acquire(val)) )
            return rc;

        if ( (rc=read_data()) )
            return rc;


    }
    return 0;
}

int LaserSyncRun::acquire(unsigned int val)
{
    ConfigFactory *cfg = ConfigFactory::instance();
    LaserSyncConfig *lc = dynamic_cast<LaserSyncConfig *>(cfg->get("lasersync"));

    return get_daq_manager()->get_board()->laser_sync(val, lc->nevts());
}
