/*
 * RunManager.h
 *
 * This an abstract class that defines the operations to be performed
 * by each run mode (Laser, Laser Sync, Pedestal, Rad source, Calibration)
 * at each stage of the DAQ process
 *
 *  Created on: May 12, 2010
 *      Author: lacasta
 */

#ifndef RUNMANAGER_H_
#define RUNMANAGER_H_

#include <iosfwd>
#include <DataFormat.h>
#include <Alibava.h>
#include <Plugin.h>

class DAQmanager;

class RunManager: public DataReceiver
{
    public:
        enum RunType
        {
            Unknown=0,
            Calibration=1,
            LaserSync,
            Laser,
            RadSource,
            Pedestal,
            ChargeScan,
            LastRType
        };

    protected:
        unsigned int _ntrig; // Number of triggers counted
        DAQmanager *daqm;
        Plugin *_plugin;


    public:
        RunManager(DAQmanager *d);
        virtual ~RunManager() {}

        void set_daq_manager( DAQmanager *a)
        {
            daqm = a;
        }
        DAQmanager *get_daq_manager() { return daqm; }


        void set_plugin(Plugin *p);

        Plugin *plugin() { return _plugin; }

        // The receiver part
        void receive(const EventData *) {}

        // Prepares to start the run
        virtual void start_of_run() {}

        // Takes data
        // The input is an ostream where the RUnmanager can write
        // specific information.
        // Returns 0 for a successful run and != otherwise
        virtual int loop(std::ostream &header) = 0;

        // Instruct Alibava to acquire events and store them
        // locally
        virtual int acquire(unsigned int) = 0;

        // Read data from Alibava
        virtual int read_data() = 0;

        virtual void check_new_event(int ievt);

        static RunManager *factory(RunType, DAQmanager *);

        /*
         * Getters and setters for ntrig. This is the number of triggers
         * counted by the firmware
         */
        unsigned int  getNtrig() const;
        void setNtrig(unsigned int ntrig);
        unsigned int  incNtrig(unsigned int val);
};

#endif /* RUNMANAGER_H_ */
